@@ -1,16 +1,23 @@
 import warnings
 
-from pyramid.interfaces import IRequest
-from pyramid.interfaces import IRouteRequest
-from pyramid.interfaces import IRoutesMapper
-from pyramid.interfaces import PHASE2_CONFIG
+from pyramid.interfaces import (
+    IRequest,
+    IRouteRequest,
+    IRoutesMapper,
+    PHASE2_CONFIG,
+    )
 
 from pyramid.exceptions import ConfigurationError
+from pyramid.registry import predvalseq
 from pyramid.request import route_request_iface
 from pyramid.urldispatch import RoutesMapper
 
-from pyramid.config.util import action_method
-from pyramid.config.util import make_predicates
+from pyramid.config.util import (
+    action_method,
+    as_sorted_tuple,
+    )
+
+import pyramid.config.predicates
 
 class RoutesConfiguratorMixin(object):
     @action_method
@@ -23,7 +30,7 @@ class RoutesConfiguratorMixin(object):
                   factory=None,
                   for_=None,
                   header=None,
-                  xhr=False,
+                  xhr=None,
                   accept=None,
                   path_info=None,
                   request_method=None,
@@ -39,7 +46,7 @@ class RoutesConfiguratorMixin(object):
                   path=None,
                   pregenerator=None,
                   static=False,
-                  ):
+                  **predicates):
         """ Add a :term:`route configuration` to the current
         configuration state, as well as possibly a :term:`view
         configuration` to be used to specify a :term:`view callable`
@@ -102,7 +109,7 @@ class RoutesConfiguratorMixin(object):
           remainder marker in its pattern (see
           :ref:`using_traverse_in_a_route_pattern`).  The ``traverse``
           argument to add_route allows you to associate route patterns
-          with an arbitrary traversal path without using a a
+          with an arbitrary traversal path without using a
           ``*traverse`` remainder marker; instead you can use other
           match information.
 
@@ -231,6 +238,19 @@ class RoutesConfiguratorMixin(object):
           request, this predicate will be true.  If this predicate
           returns ``False``, route matching continues.
 
+        effective_principals
+
+          If specified, this value should be a :term:`principal` identifier or
+          a sequence of principal identifiers.  If the
+          :func:`pyramid.security.effective_principals` method indicates that
+          every principal named in the argument list is present in the current
+          request, this predicate will return True; otherwise it will return
+          False.  For example:
+          ``effective_principals=pyramid.security.Authenticated`` or
+          ``effective_principals=('fred', 'group:admins')``.
+
+          .. versionadded:: 1.4a4
+
         custom_predicates
 
           This value should be a sequence of references to custom
@@ -249,6 +269,15 @@ class RoutesConfiguratorMixin(object):
           :ref:`custom_route_predicates` for more information about
           ``info``.
 
+        predicates
+
+          Pass a key/value pair here to use a third-party predicate
+          registered via
+          :meth:`pyramid.config.Configurator.add_view_predicate`.  More than
+          one key/value pair can be used at the same time.  See
+          :ref:`view_and_route_predicates` for more information about
+          third-party predicates.  This argument is new as of Pyramid 1.4.
+          
         View-Related Arguments
 
         .. warning::
@@ -343,16 +372,8 @@ class RoutesConfiguratorMixin(object):
         """
         # these are route predicates; if they do not match, the next route
         # in the routelist will be tried
-        ignored, predicates, ignored = make_predicates(
-            xhr=xhr,
-            request_method=request_method,
-            path_info=path_info,
-            request_param=request_param,
-            header=header,
-            accept=accept,
-            traverse=traverse,
-            custom=custom_predicates
-            )
+        if request_method is not None:
+            request_method = as_sorted_tuple(request_method)
 
         factory = self.maybe_dotted(factory)
         if pattern is None:
@@ -365,6 +386,38 @@ class RoutesConfiguratorMixin(object):
 
         mapper = self.get_routes_mapper()
 
+        introspectables = []
+
+        intr = self.introspectable('routes',
+                                   name,
+                                   '%s (pattern: %r)' % (name, pattern),
+                                   'route')
+        intr['name'] = name
+        intr['pattern'] = pattern
+        intr['factory'] = factory
+        intr['xhr'] = xhr
+        intr['request_methods'] = request_method
+        intr['path_info'] = path_info
+        intr['request_param'] = request_param
+        intr['header'] = header
+        intr['accept'] = accept
+        intr['traverse'] = traverse
+        intr['custom_predicates'] = custom_predicates
+        intr['pregenerator'] = pregenerator
+        intr['static'] = static
+        intr['use_global_views'] = use_global_views
+        introspectables.append(intr)
+
+        if factory:
+            factory_intr = self.introspectable('root factories',
+                                               name,
+                                               self.object_description(factory),
+                                               'root factory')
+            factory_intr['factory'] = factory
+            factory_intr['route_name'] = name
+            factory_intr.relate('routes', name)
+            introspectables.append(factory_intr)
+
         def register_route_request_iface():
             request_iface = self.registry.queryUtility(IRouteRequest, name=name)
             if request_iface is None:
@@ -377,9 +430,28 @@ class RoutesConfiguratorMixin(object):
                     request_iface, IRouteRequest, name=name)
 
         def register_connect():
-            return mapper.connect(name, pattern, factory, predicates=predicates,
-                                  pregenerator=pregenerator, static=static)
+            pvals = predicates.copy()
+            pvals.update(
+                dict(
+                    xhr=xhr,
+                    request_method=request_method,
+                    path_info=path_info,
+                    request_param=request_param,
+                    header=header,
+                    accept=accept,
+                    traverse=traverse,
+                    custom=predvalseq(custom_predicates),
+                    )
+                )
 
+            predlist = self.get_predlist('route')
+            _, preds, _ = predlist.make(self, **pvals)
+            route = mapper.connect(
+                name, pattern, factory, predicates=preds,
+                pregenerator=pregenerator, static=static
+                )
+            intr['object'] = route
+            return route
 
         # We have to connect routes in the order they were provided;
         # we can't use a phase to do that, because when the actions are
@@ -389,7 +461,7 @@ class RoutesConfiguratorMixin(object):
         # But IRouteRequest interfaces must be registered before we begin to
         # process view registrations (in phase 3)
         self.action(('route', name), register_route_request_iface,
-                    order=PHASE2_CONFIG)
+                    order=PHASE2_CONFIG, introspectables=introspectables)
 
         # deprecated adding views from add_route; must come after
         # route registration for purposes of autocommit ordering
@@ -404,6 +476,48 @@ class RoutesConfiguratorMixin(object):
                 attr=view_attr,
             )
 
+    @action_method
+    def add_route_predicate(self, name, factory, weighs_more_than=None,
+                           weighs_less_than=None):
+        """ Adds a route predicate factory.  The view predicate can later be
+        named as a keyword argument to
+        :meth:`pyramid.config.Configurator.add_route`.
+
+        ``name`` should be the name of the predicate.  It must be a valid
+        Python identifier (it will be used as a keyword argument to
+        ``add_view``).
+
+        ``factory`` should be a :term:`predicate factory`.
+
+        See :ref:`view_and_route_predicates` for more information.
+
+        .. note::
+
+           This method is new as of Pyramid 1.4.
+        """
+        self._add_predicate(
+            'route',
+            name,
+            factory,
+            weighs_more_than=weighs_more_than,
+            weighs_less_than=weighs_less_than
+            )
+
+    def add_default_route_predicates(self):
+        p = pyramid.config.predicates
+        for (name, factory) in (
+            ('xhr', p.XHRPredicate),
+            ('request_method', p.RequestMethodPredicate),
+            ('path_info', p.PathInfoPredicate),
+            ('request_param', p.RequestParamPredicate),
+            ('header', p.HeaderPredicate),
+            ('accept', p.AcceptPredicate),
+            ('effective_principals', p.EffectivePrincipalsPredicate),
+            ('custom', p.CustomPredicate),
+            ('traverse', p.TraversePredicate),
+            ):
+            self.add_route_predicate(name, factory)
+    
     def get_routes_mapper(self):
         """ Return the :term:`routes mapper` object associated with
         this configurator's :term:`registry`."""
