@@ -1,19 +1,26 @@
 from zope.interface import Interface
 
-from pyramid.interfaces import ITraverser
-from pyramid.interfaces import IAuthorizationPolicy
-from pyramid.interfaces import IAuthenticationPolicy
-from pyramid.interfaces import IRendererFactory
+from pyramid.interfaces import (
+    ITraverser,
+    IAuthorizationPolicy,
+    IAuthenticationPolicy,
+    IRendererFactory,
+    )
 
 from pyramid.renderers import RendererHelper
-from pyramid.traversal import traversal_path_info
 
-from pyramid.config.util import action_method
+from pyramid.traversal import (
+    decode_path_info,
+    split_path_info,
+    )
+
+from pyramid.util import action_method
 
 class TestingConfiguratorMixin(object):
     # testing API
     def testing_securitypolicy(self, userid=None, groupids=(),
-                               permissive=True):
+                               permissive=True, remember_result=None,
+                               forget_result=None):
         """Unit/integration testing helper: Registers a pair of faux
         :app:`Pyramid` security policies: a :term:`authentication
         policy` and a :term:`authorization policy`.
@@ -25,6 +32,24 @@ class TestingConfiguratorMixin(object):
         nonpermissive :term:`authorization policy` is registered; this
         policy denies all access.
 
+        ``remember_result``, if provided, should be the result returned by
+        the ``remember`` method of the faux authentication policy.  If it is
+        not provided (or it is provided, and is ``None``), the default value
+        ``[]`` (the empty list) will be returned by ``remember``.
+
+        .. note::
+
+           ``remember_result`` is new as of Pyramid 1.4.
+
+        ``forget_result``, if provided, should be the result returned by
+        the ``forget`` method of the faux authentication policy.  If it is
+        not provided (or it is provided, and is ``None``), the default value
+        ``[]`` (the empty list) will be returned by ``forget``.
+
+        .. note::
+
+           ``forget_result`` is new as of Pyramid 1.4.
+
         The behavior of the registered :term:`authentication policy`
         depends on the values provided for the ``userid`` and
         ``groupids`` argument.  The authentication policy will return
@@ -41,9 +66,12 @@ class TestingConfiguratorMixin(object):
         :func:`pyramid.security.principals_allowed_by_permission`.
         """
         from pyramid.testing import DummySecurityPolicy
-        policy = DummySecurityPolicy(userid, groupids, permissive)
+        policy = DummySecurityPolicy(
+            userid, groupids, permissive, remember_result, forget_result
+            )
         self.registry.registerUtility(policy, IAuthorizationPolicy)
         self.registry.registerUtility(policy, IAuthenticationPolicy)
+        return policy
 
     def testing_resources(self, resources):
         """Unit/integration testing helper: registers a dictionary of
@@ -64,9 +92,9 @@ class TestingConfiguratorMixin(object):
                 self.context = context
 
             def __call__(self, request):
-                path = request.environ['PATH_INFO']
+                path = decode_path_info(request.environ['PATH_INFO'])
                 ob = resources[path]
-                traversed = traversal_path_info(path)
+                traversed = split_path_info(path)
                 return {'context':ob, 'view_name':'','subpath':(),
                         'traversed':traversed, 'virtual_root':ob,
                         'virtual_root_path':(), 'root':ob}
