@@ -1,9 +1,13 @@
-from pyramid.httpexceptions import HTTPNotFound
-from pyramid.httpexceptions import HTTPForbidden
+from pyramid.httpexceptions import (
+    HTTPNotFound,
+    HTTPForbidden,
+    )
 
 NotFound = HTTPNotFound # bw compat
 Forbidden = HTTPForbidden # bw compat
 
+CR = '\n'
+
 class PredicateMismatch(HTTPNotFound):
     """
     Internal exception (not an API) raised by multiviews when no
@@ -37,15 +41,14 @@ class ConfigurationConflictError(ConfigurationError):
 
     def __str__(self):
         r = ["Conflicting configuration actions"]
-        items = self._conflicts.items()
-        items.sort()
+        items = sorted(self._conflicts.items())
         for discriminator, infos in items:
             r.append("  For: %s" % (discriminator, ))
             for info in infos:
-                for line in unicode(info).rstrip().split(u'\n'):
-                    r.append(u"    "+line)
+                for line in str(info).rstrip().split(CR):
+                    r.append("    "+line)
 
-        return "\n".join(r)
+        return CR.join(r)
 
 
 class ConfigurationExecutionError(ConfigurationError):
@@ -57,3 +60,21 @@ class ConfigurationExecutionError(ConfigurationError):
 
     def __str__(self):
         return "%s: %s\n  in:\n  %s" % (self.etype, self.evalue, self.info)
+
+class CyclicDependencyError(Exception):
+    """ The exception raised when the Pyramid topological sorter detects a
+    cyclic dependency."""
+    def __init__(self, cycles):
+        self.cycles = cycles
+
+    def __str__(self):
+        L = []
+        cycles = self.cycles
+        for cycle in cycles:
+            dependent = cycle
+            dependees = cycles[cycle]
+            L.append('%r sorts before %r' % (dependent, dependees))
+        msg = 'Implicit ordering cycle:' + '; '.join(L)
+        return msg
+
+
