@@ -1,20 +1,32 @@
+import json
+
 from zope.deprecation import deprecate
 from zope.deprecation.deprecation import deprecated
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface.interface import InterfaceClass
 
 from webob import BaseRequest
 
-from pyramid.interfaces import IRequest
-from pyramid.interfaces import IResponse
-from pyramid.interfaces import ISessionFactory
-from pyramid.interfaces import IResponseFactory
+from pyramid.interfaces import (
+    IRequest,
+    IResponse,
+    ISessionFactory,
+    IResponseFactory,
+    )
+
+from pyramid.compat import (
+    iterkeys_,
+    itervalues_,
+    iteritems_,
+    text_,
+    bytes_,
+    native_,
+    )
 
-from pyramid.compat import json
-from pyramid.exceptions import ConfigurationError
 from pyramid.decorator import reify
 from pyramid.response import Response
 from pyramid.url import URLMethodsMixin
+from pyramid.util import InstancePropertyMixin
 
 class TemplateContext(object):
     pass
@@ -64,15 +76,15 @@ class DeprecatedRequestMethodsMixin(object):
 
     @deprecate(dictlike)
     def iteritems(self):
-        return self.environ.iteritems()
+        return iteritems_(self.environ)
 
     @deprecate(dictlike)
     def iterkeys(self):
-        return self.environ.iterkeys()
+        return iterkeys_(self.environ)
 
     @deprecate(dictlike)
     def itervalues(self):
-        return self.environ.itervalues()
+        return itervalues_(self.environ)
 
     @deprecate(dictlike)
     def keys(self):
@@ -288,8 +300,9 @@ class CallbackMethodsMixin(object):
             callback = callbacks.pop(0)
             callback(self)
 
+@implementer(IRequest)
 class Request(BaseRequest, DeprecatedRequestMethodsMixin, URLMethodsMixin,
-              CallbackMethodsMixin):
+              CallbackMethodsMixin, InstancePropertyMixin):
     """
     A subclass of the :term:`WebOb` Request class.  An instance of
     this class is created by the :term:`router` and is provided to a
@@ -310,15 +323,17 @@ class Request(BaseRequest, DeprecatedRequestMethodsMixin, URLMethodsMixin,
     release of this :app:`Pyramid` version.  See
     http://pythonpaste.org/webob/ for further information.
     """
-    implements(IRequest)
     exception = None
     exc_info = None
     matchdict = None
     matched_route = None
 
+    ResponseClass = Response
+
     @reify
     def tmpl_context(self):
-        """ Template context (for Pylons apps) """
+        # docs-deprecated template context for Pylons-like apps; do not
+        # remove.
         return TemplateContext()
 
     @reify
@@ -329,7 +344,7 @@ class Request(BaseRequest, DeprecatedRequestMethodsMixin, URLMethodsMixin,
         :class:`pyramid.exceptions.ConfigurationError` will be raised"""
         factory = self.registry.queryUtility(ISessionFactory)
         if factory is None:
-            raise ConfigurationError(
+            raise AttributeError(
                 'No session factory registered '
                 '(see the Sessions chapter of the Pyramid documentation)')
         return factory(self)
@@ -357,6 +372,8 @@ class Request(BaseRequest, DeprecatedRequestMethodsMixin, URLMethodsMixin,
     def is_response(self, ob):
         """ Return ``True`` if the object passed as ``ob`` is a valid
         response object, ``False`` otherwise."""
+        if ob.__class__ is Response:
+            return True
         registry = self.registry
         adapted = registry.queryAdapterOrSelf(ob, IResponse)
         if adapted is None:
@@ -365,7 +382,7 @@ class Request(BaseRequest, DeprecatedRequestMethodsMixin, URLMethodsMixin,
 
     @property
     def json_body(self):
-        return json.loads(self.body.decode(self.charset))
+        return json.loads(text_(self.body, self.charset))
 
 def route_request_iface(name, bases=()):
     # zope.interface treats the __name__ as the __doc__ and changes __name__
@@ -409,7 +426,8 @@ def call_app_with_subpath_as_path_info(request, app):
     new_script_name = ''
 
     # compute new_path_info
-    new_path_info = '/' + '/'.join([x.encode('utf-8') for x in subpath])
+    new_path_info = '/' + '/'.join([native_(x.encode('utf-8'), 'latin-1')
+                                    for x in subpath])
 
     if new_path_info != '/': # don't want a sole double-slash
         if path_info != '/': # if orig path_info is '/', we're already done
@@ -427,7 +445,7 @@ def call_app_with_subpath_as_path_info(request, app):
             break
         el = workback.pop()
         if el:
-            tmp.insert(0, el.decode('utf-8'))
+            tmp.insert(0, text_(bytes_(el, 'latin-1'), 'utf-8'))
 
     # strip all trailing slashes from workback to avoid appending undue slashes
     # to end of script_name
@@ -439,4 +457,5 @@ def call_app_with_subpath_as_path_info(request, app):
     new_request = request.copy()
     new_request.environ['SCRIPT_NAME'] = new_script_name
     new_request.environ['PATH_INFO'] = new_path_info
+
     return new_request.get_response(app)
