@@ -1,5 +1,6 @@
 from sqlalchemy import (
     Column,
+    Index,
     Integer,
     Text,
     )
@@ -20,9 +21,11 @@ Base = declarative_base()
 class MyModel(Base):
     __tablename__ = 'models'
     id = Column(Integer, primary_key=True)
-    name = Column(Text, unique=True)
+    name = Column(Text)
     value = Column(Integer)
 
     def __init__(self, name, value):
         self.name = name
         self.value = value
+
+Index('my_index', MyModel.name, unique=True, mysql_length=255)
