@@ -163,9 +163,8 @@ class TestOverrideProvider(unittest.TestCase):
         import pyramid.tests.test_config
         provider = self._makeOne(pyramid.tests.test_config)
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, resource_name)).read()
-        result = provider.get_resource_stream(None, resource_name)
-        self.assertEqual(result.read().replace('\r', ''), expected)
+        with provider.get_resource_stream(None, resource_name) as result:
+            _assertBody(result.read(), os.path.join(here, resource_name))
 
     def test_get_resource_string_no_overrides(self):
         import os
@@ -173,9 +172,8 @@ class TestOverrideProvider(unittest.TestCase):
         import pyramid.tests.test_config
         provider = self._makeOne(pyramid.tests.test_config)
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, resource_name)).read()
         result = provider.get_resource_string(None, resource_name)
-        self.assertEqual(result.replace('\r', ''), expected)
+        _assertBody(result, os.path.join(here, resource_name))
 
     def test_has_resource_no_overrides(self):
         resource_name = 'test_assets.py'
@@ -221,9 +219,8 @@ class TestOverrideProvider(unittest.TestCase):
         import pyramid.tests.test_config
         provider = self._makeOne(pyramid.tests.test_config)
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, resource_name)).read()
-        result = provider.get_resource_stream(None, resource_name)
-        self.assertEqual(result.read(), expected)
+        with provider.get_resource_stream(None, resource_name) as result:
+            _assertBody(result.read(), os.path.join(here, resource_name))
 
     def test_get_resource_string_override_returns_None(self):
         overrides = DummyOverrides(None)
@@ -233,9 +230,8 @@ class TestOverrideProvider(unittest.TestCase):
         import pyramid.tests.test_config
         provider = self._makeOne(pyramid.tests.test_config)
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, resource_name)).read()
         result = provider.get_resource_string(None, resource_name)
-        self.assertEqual(result, expected)
+        _assertBody(result, os.path.join(here, resource_name))
 
     def test_has_resource_override_returns_None(self):
         overrides = DummyOverrides(None)
@@ -273,12 +269,13 @@ class TestOverrideProvider(unittest.TestCase):
         self.assertEqual(result, 'value')
 
     def test_get_resource_stream_override_returns_value(self):
-        overrides = DummyOverrides('value')
+        from io import BytesIO
+        overrides = DummyOverrides(BytesIO(b'value'))
         import pyramid.tests.test_config
         self._registerOverrides(overrides)
         provider = self._makeOne(pyramid.tests.test_config)
-        result = provider.get_resource_stream(None, 'test_assets.py')
-        self.assertEqual(result, 'value')
+        with provider.get_resource_stream(None, 'test_assets.py') as stream:
+            self.assertEqual(stream.getvalue(), b'value')
 
     def test_get_resource_string_override_returns_value(self):
         overrides = DummyOverrides('value')
@@ -317,16 +314,40 @@ class TestPackageOverrides(unittest.TestCase):
         from pyramid.config.assets import PackageOverrides
         return PackageOverrides
 
-    def _makeOne(self, package, pkg_resources=None):
+    def _makeOne(self, package=None, pkg_resources=None):
+        if package is None:
+            package = DummyPackage('package')
         klass = self._getTargetClass()
         if pkg_resources is None:
             pkg_resources = DummyPkgResources()
         return klass(package, pkg_resources=pkg_resources)
 
+    def test_class_conforms_to_IPackageOverrides(self):
+        from zope.interface.verify import verifyClass
+        from pyramid.interfaces import IPackageOverrides
+        verifyClass(IPackageOverrides, self._getTargetClass())
+
+    def test_instance_conforms_to_IPackageOverrides(self):
+        from zope.interface.verify import verifyObject
+        from pyramid.interfaces import IPackageOverrides
+        verifyObject(IPackageOverrides, self._makeOne())
+
+    def test_class_conforms_to_IPEP302Loader(self):
+        from zope.interface.verify import verifyClass
+        from pyramid.interfaces import IPEP302Loader
+        verifyClass(IPEP302Loader, self._getTargetClass())
+
+    def test_instance_conforms_to_IPEP302Loader(self):
+        from zope.interface.verify import verifyObject
+        from pyramid.interfaces import IPEP302Loader
+        verifyObject(IPEP302Loader, self._makeOne())
+
     def test_ctor_package_already_has_loader_of_different_type(self):
         package = DummyPackage('package')
-        package.__loader__ = True
-        self.assertRaises(TypeError, self._makeOne, package)
+        loader = package.__loader__ = DummyLoader()
+        po = self._makeOne(package)
+        self.assertTrue(package.__loader__ is po)
+        self.assertTrue(po.real_loader is loader)
 
     def test_ctor_package_already_has_loader_of_same_type(self):
         package = DummyPackage('package')
@@ -419,9 +440,8 @@ class TestPackageOverrides(unittest.TestCase):
         po = self._makeOne(package)
         po.overrides= overrides
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, 'test_assets.py')).read()
-        self.assertEqual(po.get_stream('whatever').read().replace('\r', ''),
-                         expected)
+        with po.get_stream('whatever') as stream:
+            _assertBody(stream.read(), os.path.join(here, 'test_assets.py'))
         
     def test_get_stream_file_doesnt_exist(self):
         overrides = [ DummyOverride(None), DummyOverride(
@@ -439,8 +459,8 @@ class TestPackageOverrides(unittest.TestCase):
         po = self._makeOne(package)
         po.overrides= overrides
         here = os.path.dirname(os.path.abspath(__file__))
-        expected = open(os.path.join(here, 'test_assets.py')).read()
-        self.assertEqual(po.get_string('whatever').replace('\r', ''), expected)
+        _assertBody(po.get_string('whatever'),
+                    os.path.join(here, 'test_assets.py'))
         
     def test_get_string_file_doesnt_exist(self):
         overrides = [ DummyOverride(None), DummyOverride(
@@ -506,6 +526,55 @@ class TestPackageOverrides(unittest.TestCase):
         po.overrides= overrides
         self.assertEqual(po.listdir('whatever'), None)
 
+    # PEP 302 __loader__ extensions:  use the "real" __loader__, if present.
+    def test_get_data_pkg_has_no___loader__(self):
+        package = DummyPackage('package')
+        po = self._makeOne(package)
+        self.assertRaises(NotImplementedError, po.get_data, 'whatever')
+
+    def test_get_data_pkg_has___loader__(self):
+        package = DummyPackage('package')
+        loader = package.__loader__  = DummyLoader()
+        po = self._makeOne(package)
+        self.assertEqual(po.get_data('whatever'), b'DEADBEEF')
+        self.assertEqual(loader._got_data, 'whatever')
+
+    def test_is_package_pkg_has_no___loader__(self):
+        package = DummyPackage('package')
+        po = self._makeOne(package)
+        self.assertRaises(NotImplementedError, po.is_package, 'whatever')
+
+    def test_is_package_pkg_has___loader__(self):
+        package = DummyPackage('package')
+        loader = package.__loader__  = DummyLoader()
+        po = self._makeOne(package)
+        self.assertTrue(po.is_package('whatever'))
+        self.assertEqual(loader._is_package, 'whatever')
+
+    def test_get_code_pkg_has_no___loader__(self):
+        package = DummyPackage('package')
+        po = self._makeOne(package)
+        self.assertRaises(NotImplementedError, po.get_code, 'whatever')
+
+    def test_get_code_pkg_has___loader__(self):
+        package = DummyPackage('package')
+        loader = package.__loader__  = DummyLoader()
+        po = self._makeOne(package)
+        self.assertEqual(po.get_code('whatever'), b'DEADBEEF')
+        self.assertEqual(loader._got_code, 'whatever')
+
+    def test_get_source_pkg_has_no___loader__(self):
+        package = DummyPackage('package')
+        po = self._makeOne(package)
+        self.assertRaises(NotImplementedError, po.get_source, 'whatever')
+
+    def test_get_source_pkg_has___loader__(self):
+        package = DummyPackage('package')
+        loader = package.__loader__  = DummyLoader()
+        po = self._makeOne(package)
+        self.assertEqual(po.get_source('whatever'), 'def foo():\n    pass')
+        self.assertEqual(loader._got_source, 'whatever')
+
 class TestDirectoryOverride(unittest.TestCase):
     def _getTargetClass(self):
         from pyramid.config.assets import DirectoryOverride
@@ -574,16 +643,44 @@ class DummyPkgResources:
 
     def register_loader_type(self, typ, inst):
         self.registered.append((typ, inst))
-        
+
 class DummyPackage:
     def __init__(self, name):
         self.__name__ = name
+ 
+class DummyLoader:
+    _got_data = _is_package = None
+    def get_data(self, path):
+        self._got_data = path
+        return b'DEADBEEF'
+    def is_package(self, fullname):
+        self._is_package = fullname
+        return True
+    def get_code(self, fullname):
+        self._got_code = fullname
+        return b'DEADBEEF'
+    def get_source(self, fullname):
+        self._got_source = fullname
+        return 'def foo():\n    pass'
 
 class DummyUnderOverride:
     def __call__(self, package, path, override_package, override_prefix,
-                 _info=u''):
+                 _info=''):
         self.package = package
         self.path = path
         self.override_package = override_package
         self.override_prefix = override_prefix
 
+def read_(src):
+    with open(src, 'rb') as f:
+        contents = f.read()
+    return contents
+
+def _assertBody(body, filename):
+    # strip both \n and \r for windows
+    body = body.replace(b'\r', b'')
+    body = body.replace(b'\n', b'')
+    data = read_(filename)
+    data = data.replace(b'\r', b'')
+    data = data.replace(b'\n', b'')
+    assert(body == data)
