@@ -2,6 +2,7 @@ import unittest
 
 from pyramid.testing import cleanUp
 from pyramid import testing
+from pyramid.compat import text_
 
 class TestTemplateRendererFactory(unittest.TestCase):
     def setUp(self):
@@ -291,7 +292,37 @@ class TestChameleonRendererLookup(unittest.TestCase):
         if path.endswith('.pyc'): # pragma: no cover
             path = path[:-1]
         self.assertTrue(factory.path.startswith(path))
-        self.assertEqual(factory.kw, {})
+        self.assertEqual(factory.kw, {'macro':None})
+
+    def test___call__spec_withmacro(self):
+        from pyramid.interfaces import ITemplateRenderer
+        import os
+        from pyramid import tests
+        module_name = tests.__name__
+        relpath = 'fixtures/withmacro#foo.pt'
+        renderer = {}
+        factory = DummyFactory(renderer)
+        spec = '%s:%s' % (module_name, relpath)
+        reg = self.config.registry
+        info = DummyRendererInfo({
+            'name':spec,
+            'package':None,
+            'registry':reg,
+            'settings':{},
+            'type':'type',
+            })
+        lookup = self._makeOne(factory)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
+        path = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            'fixtures',
+            'withmacro.pt')
+        self.assertTrue(factory.path.startswith(path))
+        self.assertEqual(factory.kw, {'macro':'foo'})
+        self.assertTrue(
+            reg.getUtility(ITemplateRenderer, name=spec) is renderer
+            )
 
     def test___call__reload_assets_true(self):
         import pyramid.tests
@@ -339,93 +370,96 @@ class TestChameleonRendererLookup(unittest.TestCase):
         self.assertNotEqual(reg.queryUtility(ITemplateRenderer, name=spec),
                             None)
 
-class TestRendererFromName(unittest.TestCase):
-    def setUp(self):
-        from zope.deprecation import __show__
-        __show__.off()
-        self.config = cleanUp()
-
-    def tearDown(self):
-        cleanUp()
-        from zope.deprecation import __show__
-        __show__.on()
-
-    def _callFUT(self, path, package=None):
-        from pyramid.renderers import renderer_from_name
-        return renderer_from_name(path, package)
-
-    def test_it(self):
-        registry = self.config.registry
-        settings = {}
-        registry.settings = settings
-        from pyramid.interfaces import IRendererFactory
-        import os
-        here = os.path.dirname(os.path.abspath(__file__))
-        fixture = os.path.join(here, 'fixtures/minimal.pt')
-        def factory(info, **kw):
-            return info
-        self.config.registry.registerUtility(
-            factory, IRendererFactory, name='.pt')
-        result = self._callFUT(fixture)
-        self.assertEqual(result.registry, registry)
-        self.assertEqual(result.type, '.pt')
-        self.assertEqual(result.package, None)
-        self.assertEqual(result.name, fixture)
-        self.assertEqual(result.settings, settings)
-
-    def test_it_with_package(self):
-        import pyramid
-        registry = self.config.registry
-        settings = {}
-        registry.settings = settings
-        from pyramid.interfaces import IRendererFactory
-        import os
-        here = os.path.dirname(os.path.abspath(__file__))
-        fixture = os.path.join(here, 'fixtures/minimal.pt')
-        def factory(info, **kw):
-            return info
-        self.config.registry.registerUtility(
-            factory, IRendererFactory, name='.pt')
-        result = self._callFUT(fixture, pyramid)
-        self.assertEqual(result.registry, registry)
-        self.assertEqual(result.type, '.pt')
-        self.assertEqual(result.package, pyramid)
-        self.assertEqual(result.name, fixture)
-        self.assertEqual(result.settings, settings)
-
-    def test_it_no_renderer(self):
-        self.assertRaises(ValueError, self._callFUT, 'foo')
-        
-
-class Test_json_renderer_factory(unittest.TestCase):
+class TestJSON(unittest.TestCase):
     def setUp(self):
         self.config = testing.setUp()
 
     def tearDown(self):
         testing.tearDown()
-        
-    def _callFUT(self, name):
-        from pyramid.renderers import json_renderer_factory
-        return json_renderer_factory(name)
+
+    def _makeOne(self, **kw):
+        from pyramid.renderers import JSON
+        return JSON(**kw)
 
     def test_it(self):
-        renderer = self._callFUT(None)
+        renderer = self._makeOne()(None)
         result = renderer({'a':1}, {})
         self.assertEqual(result, '{"a": 1}')
 
     def test_with_request_content_type_notset(self):
         request = testing.DummyRequest()
-        renderer = self._callFUT(None)
+        renderer = self._makeOne()(None)
         renderer({'a':1}, {'request':request})
         self.assertEqual(request.response.content_type, 'application/json')
 
     def test_with_request_content_type_set(self):
         request = testing.DummyRequest()
         request.response.content_type = 'text/mishmash'
-        renderer = self._callFUT(None)
+        renderer = self._makeOne()(None)
         renderer({'a':1}, {'request':request})
         self.assertEqual(request.response.content_type, 'text/mishmash')
 
+    def test_with_custom_adapter(self):
+        request = testing.DummyRequest()
+        from datetime import datetime
+        def adapter(obj, req):
+            self.assertEqual(req, request)
+            return obj.isoformat()
+        now = datetime.utcnow()
+        renderer = self._makeOne()
+        renderer.add_adapter(datetime, adapter)
+        result = renderer(None)({'a':now}, {'request':request})
+        self.assertEqual(result, '{"a": "%s"}' % now.isoformat())
+
+    def test_with_custom_adapter2(self):
+        request = testing.DummyRequest()
+        from datetime import datetime
+        def adapter(obj, req):
+            self.assertEqual(req, request)
+            return obj.isoformat()
+        now = datetime.utcnow()
+        renderer = self._makeOne(adapters=((datetime, adapter),))
+        result = renderer(None)({'a':now}, {'request':request})
+        self.assertEqual(result, '{"a": "%s"}' % now.isoformat())
+
+    def test_with_custom_serializer(self):
+        class Serializer(object):
+            def __call__(self, obj, **kw):
+                self.obj = obj
+                self.kw = kw
+                return 'foo'
+        serializer = Serializer()
+        renderer = self._makeOne(serializer=serializer, baz=5)
+        obj = {'a':'b'}
+        result = renderer(None)(obj, {})
+        self.assertEqual(result, 'foo')
+        self.assertEqual(serializer.obj, obj)
+        self.assertEqual(serializer.kw['baz'], 5)
+        self.assertTrue('default' in serializer.kw)
+
+    def test_with_object_adapter(self):
+        request = testing.DummyRequest()
+        outerself = self
+        class MyObject(object):
+            def __init__(self, x):
+                self.x = x
+            def __json__(self, req):
+                outerself.assertEqual(req, request)
+                return {'x': self.x}
+
+        objects = [MyObject(1), MyObject(2)]
+        renderer = self._makeOne()(None)
+        result = renderer(objects, {'request':request})
+        self.assertEqual(result, '[{"x": 1}, {"x": 2}]')
+
+    def test_with_object_adapter_no___json__(self):
+        class MyObject(object):
+            def __init__(self, x):
+                self.x = x
+        objects = [MyObject(1), MyObject(2)]
+        renderer = self._makeOne()(None)
+        self.assertRaises(TypeError, renderer, objects, {})
+
 class Test_string_renderer_factory(unittest.TestCase):
     def _callFUT(self, name):
         from pyramid.renderers import string_renderer_factory
@@ -433,7 +467,7 @@ class Test_string_renderer_factory(unittest.TestCase):
 
     def test_it_unicode(self):
         renderer = self._callFUT(None)
-        value = unicode('La Pe\xc3\xb1a', 'utf-8')
+        value = text_('La Pe\xc3\xb1a', 'utf-8')
         result = renderer(value, {})
         self.assertEqual(result, value)
                           
@@ -452,14 +486,14 @@ class Test_string_renderer_factory(unittest.TestCase):
     def test_with_request_content_type_notset(self):
         request = testing.DummyRequest()
         renderer = self._callFUT(None)
-        renderer(None, {'request':request})
+        renderer('', {'request':request})
         self.assertEqual(request.response.content_type, 'text/plain')
 
     def test_with_request_content_type_set(self):
         request = testing.DummyRequest()
         request.response.content_type = 'text/mishmash'
         renderer = self._callFUT(None)
-        renderer(None, {'request':request})
+        renderer('', {'request':request})
         self.assertEqual(request.response.content_type, 'text/mishmash')
 
 
@@ -504,6 +538,7 @@ class TestRendererHelper(unittest.TestCase):
         def renderer(*arg):
             def respond(*arg):
                 return arg
+            renderer.respond = respond
             return respond
         self.config.registry.registerUtility(renderer, IRendererFactory,
                                              name='.foo')
@@ -525,6 +560,11 @@ class TestRendererHelper(unittest.TestCase):
         self.assertEqual(response.body[0], 'values')
         self.assertEqual(response.body[1], {})
 
+    def test_get_renderer(self):
+        factory = self._registerRendererFactory()
+        helper = self._makeOne('loo.foo')
+        self.assertEqual(helper.get_renderer(), factory.respond)
+
     def test_render_view(self):
         self._registerRendererFactory()
         self._registerResponseFactory()
@@ -541,7 +581,8 @@ class TestRendererHelper(unittest.TestCase):
                            'renderer_name': 'loo.foo',
                            'request': request,
                            'context': 'context',
-                           'view': 'view'}
+                           'view': 'view',
+                           'req': request,}
                          )
 
     def test_render_explicit_registry(self):
@@ -574,7 +615,8 @@ class TestRendererHelper(unittest.TestCase):
                   'context':context,
                   'renderer_name':'loo.foo',
                   'view':None,
-                  'renderer_info':helper
+                  'renderer_info':helper,
+                  'req':request,
                   }
         self.assertEqual(result[0], 'values')
         self.assertEqual(result[1], system)
@@ -593,22 +635,22 @@ class TestRendererHelper(unittest.TestCase):
         request = None
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_request_is_None_response_factory_exists(self):
         self._registerResponseFactory()
         request = None
         helper = self._makeOne('loo.foo')
-        response = helper._make_response('abc', request)
+        response = helper._make_response(b'abc', request)
         self.assertEqual(response.__class__.__name__, 'ResponseFactory')
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_result_is_unicode(self):
         from pyramid.response import Response
         request = testing.DummyRequest()
         request.response = Response()
         helper = self._makeOne('loo.foo')
-        la = unicode('/La Pe\xc3\xb1a', 'utf-8')
+        la = text_('/La Pe\xc3\xb1a', 'utf-8')
         response = helper._make_response(la, request)
         self.assertEqual(response.body, la.encode('utf-8'))
 
@@ -617,10 +659,28 @@ class TestRendererHelper(unittest.TestCase):
         request = testing.DummyRequest()
         request.response = Response()
         helper = self._makeOne('loo.foo')
-        la = unicode('/La Pe\xc3\xb1a', 'utf-8')
+        la = text_('/La Pe\xc3\xb1a', 'utf-8')
         response = helper._make_response(la.encode('utf-8'), request)
         self.assertEqual(response.body, la.encode('utf-8'))
 
+    def test__make_response_result_is_None_no_body(self):
+        from pyramid.response import Response
+        request = testing.DummyRequest()
+        request.response = Response()
+        helper = self._makeOne('loo.foo')
+        response = helper._make_response(None, request)
+        self.assertEqual(response.body, b'')
+
+    def test__make_response_result_is_None_existing_body_not_molested(self):
+        from pyramid.response import Response
+        request = testing.DummyRequest()
+        response = Response()
+        response.body = b'abc'
+        request.response = response
+        helper = self._makeOne('loo.foo')
+        response = helper._make_response(None, request)
+        self.assertEqual(response.body, b'abc')
+        
     def test__make_response_with_content_type(self):
         from pyramid.response import Response
         request = testing.DummyRequest()
@@ -630,7 +690,7 @@ class TestRendererHelper(unittest.TestCase):
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.content_type, 'text/nonsense')
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_with_headerlist(self):
         from pyramid.response import Response
@@ -645,7 +705,7 @@ class TestRendererHelper(unittest.TestCase):
                           ('Content-Length', '3'),
                           ('a', '1'),
                           ('b', '2')])
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_with_status(self):
         from pyramid.response import Response
@@ -656,7 +716,7 @@ class TestRendererHelper(unittest.TestCase):
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.status, '406 You Lose')
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_with_charset(self):
         from pyramid.response import Response
@@ -686,9 +746,9 @@ class TestRendererHelper(unittest.TestCase):
         self.config.registry.registerUtility(ResponseFactory, IResponseFactory)
         request = testing.DummyRequest()
         helper = self._makeOne('loo.foo')
-        response = helper._make_response('abc', request)
+        response = helper._make_response(b'abc', request)
         self.assertEqual(response.__class__, ResponseFactory)
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test__make_response_with_real_request(self):
         # functional
@@ -699,7 +759,7 @@ class TestRendererHelper(unittest.TestCase):
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.status, '406 You Lose')
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
 
     def test_clone_noargs(self):
         helper = self._makeOne('name', 'package', 'registry')
@@ -718,6 +778,49 @@ class TestRendererHelper(unittest.TestCase):
         self.assertEqual(cloned_helper.registry, 'registry2')
         self.assertFalse(helper is cloned_helper)
 
+    def test_renderer_absolute_file(self):
+        registry = self.config.registry
+        settings = {}
+        registry.settings = settings
+        from pyramid.interfaces import IRendererFactory
+        import os
+        here = os.path.dirname(os.path.abspath(__file__))
+        fixture = os.path.join(here, 'fixtures/minimal.pt')
+        def factory(info, **kw):
+            return info
+        self.config.registry.registerUtility(
+            factory, IRendererFactory, name='.pt')
+        result = self._makeOne(fixture).renderer
+        self.assertEqual(result.registry, registry)
+        self.assertEqual(result.type, '.pt')
+        self.assertEqual(result.package, None)
+        self.assertEqual(result.name, fixture)
+        self.assertEqual(result.settings, settings)
+
+    def test_renderer_with_package(self):
+        import pyramid
+        registry = self.config.registry
+        settings = {}
+        registry.settings = settings
+        from pyramid.interfaces import IRendererFactory
+        import os
+        here = os.path.dirname(os.path.abspath(__file__))
+        fixture = os.path.join(here, 'fixtures/minimal.pt')
+        def factory(info, **kw):
+            return info
+        self.config.registry.registerUtility(
+            factory, IRendererFactory, name='.pt')
+        result = self._makeOne(fixture, pyramid).renderer
+        self.assertEqual(result.registry, registry)
+        self.assertEqual(result.type, '.pt')
+        self.assertEqual(result.package, pyramid)
+        self.assertEqual(result.name, fixture)
+        self.assertEqual(result.settings, settings)
+
+    def test_renderer_missing(self):
+        inst = self._makeOne('foo')
+        self.assertRaises(ValueError, getattr, inst, 'renderer')
+
 class TestNullRendererHelper(unittest.TestCase):
     def setUp(self):
         self.config = cleanUp()
@@ -811,7 +914,7 @@ class Test_render_to_response(unittest.TestCase):
             'pyramid.tests:abc/def.pt')
         renderer.string_response = 'abc'
         response = self._callFUT('abc/def.pt', dict(a=1))
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
         renderer.assert_(a=1)
         renderer.assert_(request=None)
         
@@ -822,7 +925,7 @@ class Test_render_to_response(unittest.TestCase):
         request = testing.DummyRequest()
         response = self._callFUT('abc/def.pt',
                                  dict(a=1), request=request)
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
         renderer.assert_(a=1)
         renderer.assert_(request=request)
 
@@ -834,7 +937,7 @@ class Test_render_to_response(unittest.TestCase):
         request = testing.DummyRequest()
         response = self._callFUT('abc/def.pt', dict(a=1), request=request,
                                  package=pyramid.tests)
-        self.assertEqual(response.body, 'abc')
+        self.assertEqual(response.body, b'abc')
         renderer.assert_(a=1)
         renderer.assert_(request=request)
 
