@@ -24,7 +24,7 @@ class TestRouter(unittest.TestCase):
         if mapper is None:
             mapper = RoutesMapper()
             self.registry.registerUtility(mapper, IRoutesMapper)
-        mapper.connect(name, path, factory)
+        return mapper.connect(name, path, factory)
 
     def _registerLogger(self):
         from pyramid.interfaces import IDebugLogger
@@ -177,7 +177,7 @@ class TestRouter(unittest.TestCase):
             return Response(s)
         router.registry.registerAdapter(make_response, (str,), IResponse)
         app_iter = router(environ, start_response)
-        self.assertEqual(app_iter, ['abc'])
+        self.assertEqual(app_iter, [b'abc'])
         self.assertEqual(start_response.status, '200 OK')
         self.assertEqual(environ['handled'], ['two', 'one'])
 
@@ -188,8 +188,8 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertTrue('/' in why[0], why)
-        self.assertFalse('debug_notfound' in why[0])
+        self.assertTrue('/' in why.args[0], why)
+        self.assertFalse('debug_notfound' in why.args[0])
         self.assertEqual(len(logger.messages), 0)
 
     def test_traverser_raises_notfound_class(self):
@@ -209,7 +209,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertTrue('foo' in why[0], why)
+        self.assertTrue('foo' in why.args[0], why)
 
     def test_traverser_raises_forbidden_class(self):
         from pyramid.httpexceptions import HTTPForbidden
@@ -229,7 +229,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPForbidden, router, environ, start_response)
-        self.assertTrue('foo' in why[0], why)
+        self.assertTrue('foo' in why.args[0], why)
 
     def test_call_no_view_registered_no_isettings(self):
         from pyramid.httpexceptions import HTTPNotFound
@@ -240,8 +240,8 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertTrue('/' in why[0], why)
-        self.assertFalse('debug_notfound' in why[0])
+        self.assertTrue('/' in why.args[0], why)
+        self.assertFalse('debug_notfound' in why.args[0])
         self.assertEqual(len(logger.messages), 0)
 
     def test_call_no_view_registered_debug_notfound_false(self):
@@ -254,8 +254,8 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertTrue('/' in why[0], why)
-        self.assertFalse('debug_notfound' in why[0])
+        self.assertTrue('/' in why.args[0], why)
+        self.assertFalse('debug_notfound' in why.args[0])
         self.assertEqual(len(logger.messages), 0)
 
     def test_call_no_view_registered_debug_notfound_true(self):
@@ -269,15 +269,15 @@ class TestRouter(unittest.TestCase):
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
         self.assertTrue(
-            "debug_notfound of url http://localhost:8080/; " in why[0])
-        self.assertTrue("view_name: '', subpath: []" in why[0])
-        self.assertTrue('http://localhost:8080' in why[0], why)
+            "debug_notfound of url http://localhost:8080/; " in why.args[0])
+        self.assertTrue("view_name: '', subpath: []" in why.args[0])
+        self.assertTrue('http://localhost:8080' in why.args[0], why)
 
         self.assertEqual(len(logger.messages), 1)
         message = logger.messages[0]
         self.assertTrue('of url http://localhost:8080' in message)
         self.assertTrue("path_info: " in message)
-        self.assertTrue('DummyContext instance at' in message)
+        self.assertTrue('DummyContext' in message)
         self.assertTrue("view_name: ''" in message)
         self.assertTrue("subpath: []" in message)
 
@@ -309,9 +309,41 @@ class TestRouter(unittest.TestCase):
             return Response(s)
         router.registry.registerAdapter(make_response, (str,), IResponse)
         app_iter = router(environ, start_response)
-        self.assertEqual(app_iter, ['abc'])
+        self.assertEqual(app_iter, [b'abc'])
         self.assertEqual(start_response.status, '200 OK')
 
+    def test_call_with_request_extensions(self):
+        from pyramid.interfaces import IViewClassifier
+        from pyramid.interfaces import IRequestExtensions
+        from pyramid.interfaces import IRequest
+        from pyramid.request import Request
+        context = DummyContext()
+        self._registerTraverserFactory(context)
+        class Extensions(object):
+            def __init__(self):
+                self.methods = {}
+                self.descriptors = {}
+        extensions = Extensions()
+        L = []
+        request = Request.blank('/')
+        request.request_iface = IRequest
+        request.registry = self.registry
+        request._set_extensions = lambda *x: L.extend(x)
+        def request_factory(environ):
+            return request
+        self.registry.registerUtility(extensions, IRequestExtensions)
+        environ = self._makeEnviron()
+        response = DummyResponse()
+        response.app_iter = ['Hello world']
+        view = DummyView(response)
+        self._registerView(self.config.derive_view(view), '',
+                           IViewClassifier, None, None)
+        router = self._makeOne()
+        router.request_factory = request_factory
+        start_response = DummyStartResponse()
+        router(environ, start_response)
+        self.assertEqual(L, [extensions])
+
     def test_call_view_registered_nonspecific_default_path(self):
         from pyramid.interfaces import IViewClassifier
         context = DummyContext()
@@ -427,7 +459,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPForbidden, router, environ, start_response)
-        self.assertEqual(why[0], 'unauthorized')
+        self.assertEqual(why.args[0], 'unauthorized')
 
     def test_call_view_raises_notfound(self):
         from zope.interface import Interface
@@ -447,7 +479,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertEqual(why[0], 'notfound')
+        self.assertEqual(why.args[0], 'notfound')
 
     def test_call_view_raises_response_cleared(self):
         from zope.interface import Interface
@@ -465,7 +497,7 @@ class TestRouter(unittest.TestCase):
             raise KeyError
         def exc_view(context, request):
             self.assertFalse(hasattr(request.response, 'a'))
-            request.response.body = 'OK'
+            request.response.body = b'OK'
             return request.response
         environ = self._makeEnviron()
         self._registerView(view, '', IViewClassifier, IRequest, IContext)
@@ -474,7 +506,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         itera = router(environ, start_response)
-        self.assertEqual(itera, ['OK'])
+        self.assertEqual(itera, [b'OK'])
 
     def test_call_request_has_response_callbacks(self):
         from zope.interface import Interface
@@ -625,7 +657,8 @@ class TestRouter(unittest.TestCase):
         root = object()
         def factory(request):
             return root
-        self._connectRoute('foo', 'archives/:action/:article', factory)
+        route = self._connectRoute('foo', 'archives/:action/:article', factory)
+        route.predicates = [DummyPredicate()]
         context = DummyContext()
         self._registerTraverserFactory(context)
         response = DummyResponse()
@@ -646,18 +679,19 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(request.context, context)
         self.assertEqual(request.root, root)
         matchdict = {'action':'action1', 'article':'article1'}
-        self.assertEqual(environ['bfg.routes.matchdict'], matchdict)
-        self.assertEqual(environ['bfg.routes.route'].name, 'foo')
         self.assertEqual(request.matchdict, matchdict)
         self.assertEqual(request.matched_route.name, 'foo')
-
         self.assertEqual(len(logger.messages), 1)
         self.assertTrue(
             logger.messages[0].startswith(
             "route matched for url http://localhost:8080"
             "/archives/action1/article1; "
             "route_name: 'foo', "
-            "path_info: "))
+            "path_info: ")
+            )
+        self.assertTrue(
+            "predicates: 'predicate'" in logger.messages[0]
+            )
 
     def test_call_route_match_miss_debug_routematch(self):
         from pyramid.httpexceptions import HTTPNotFound
@@ -713,8 +747,6 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(request.context, context)
         self.assertEqual(request.root, root)
         matchdict = {'action':'action1', 'article':'article1'}
-        self.assertEqual(environ['bfg.routes.matchdict'], matchdict)
-        self.assertEqual(environ['bfg.routes.route'].name, 'foo')
         self.assertEqual(request.matchdict, matchdict)
         self.assertEqual(request.matched_route.name, 'foo')
         self.assertTrue(IFoo.providedBy(request))
@@ -735,7 +767,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPNotFound, router, environ, start_response)
-        self.assertTrue('from root factory' in why[0])
+        self.assertTrue('from root factory' in why.args[0])
 
     def test_root_factory_raises_forbidden(self):
         from pyramid.interfaces import IRootFactory
@@ -753,7 +785,7 @@ class TestRouter(unittest.TestCase):
         router = self._makeOne()
         start_response = DummyStartResponse()
         why = exc_raised(HTTPForbidden, router, environ, start_response)
-        self.assertTrue('from root factory' in why[0])
+        self.assertTrue('from root factory' in why.args[0])
 
     def test_root_factory_exception_propagating(self):
         from pyramid.interfaces import IRootFactory
@@ -1132,6 +1164,12 @@ class TestRouter(unittest.TestCase):
         start_response = DummyStartResponse()
         self.assertRaises(RuntimeError, router, environ, start_response)
 
+class DummyPredicate(object):
+    def __call__(self, info, request):
+        return True
+    def text(self):
+        return 'predicate'
+
 class DummyContext:
     pass
 
@@ -1162,10 +1200,10 @@ class DummyStartResponse:
         self.headers = headers
 
 from pyramid.interfaces import IResponse
-from zope.interface import implements
+from zope.interface import implementer
 
+@implementer(IResponse)
 class DummyResponse(object):
-    implements(IResponse)
     headerlist = ()
     app_iter = ()
     environ = None
@@ -1202,7 +1240,7 @@ class DummyLogger:
 def exc_raised(exc, func, *arg, **kw):
     try:
         func(*arg, **kw)
-    except exc, e:
+    except exc as e:
         return e
     else:
         raise AssertionError('%s not raised' % exc) # pragma: no cover
