@@ -1,30 +1,4 @@
 import unittest
-from pyramid import testing
-
-class TestGetSettings(unittest.TestCase):
-    def setUp(self):
-        from pyramid.registry import Registry
-        registry = Registry('testing')
-        self.config = testing.setUp(registry=registry)
-        from zope.deprecation import __show__
-        __show__.off()
-
-    def tearDown(self):
-        self.config.end()
-        from zope.deprecation import __show__
-        __show__.on()
-        
-    def _callFUT(self):
-        from pyramid.settings import get_settings
-        return get_settings()
-
-    def test_it_nosettings(self):
-        self.assertEqual(self._callFUT()['reload_templates'], False)
-
-    def test_it_withsettings(self):
-        settings = {'a':1}
-        self.config.registry.settings = settings
-        self.assertEqual(self._callFUT(), settings)
 
 class Test_asbool(unittest.TestCase):
     def _callFUT(self, s):
@@ -81,13 +55,13 @@ class Test_aslist_cronly(unittest.TestCase):
         self.assertEqual(result, ['abc', 'def'])
 
 class Test_aslist(unittest.TestCase):
-    def _callFUT(self, val):
+    def _callFUT(self, val, **kw):
         from pyramid.settings import aslist
-        return aslist(val)
+        return aslist(val, **kw)
 
     def test_with_list(self):
         result = self._callFUT(['abc', 'def'])
-        self.assertEqual(result, ['abc', 'def'])
+        self.assertEqual(list(result), ['abc', 'def'])
         
     def test_with_string(self):
         result = self._callFUT('abc def')
@@ -100,3 +74,7 @@ class Test_aslist(unittest.TestCase):
     def test_with_string_crsep_spacesep(self):
         result = self._callFUT(' abc\n def ghi')
         self.assertEqual(result, ['abc', 'def', 'ghi'])
+
+    def test_with_string_crsep_spacesep_no_flatten(self):
+        result = self._callFUT(' abc\n def ghi ', flatten=False)
+        self.assertEqual(result, ['abc', 'def ghi'])
