@@ -1,25 +1,38 @@
+import os
 import unittest
+import warnings
 
-from pyramid.testing import setUp
-from pyramid.testing import tearDown
+from pyramid import testing
+
+from pyramid.compat import (
+    text_,
+    native_,
+    WIN,
+    )
 
 class TestURLMethodsMixin(unittest.TestCase):
     def setUp(self):
-        self.config = setUp()
+        self.config = testing.setUp()
 
     def tearDown(self):
-        tearDown()
+        testing.tearDown()
         
-    def _makeOne(self):
+    def _makeOne(self, environ=None):
         from pyramid.url import URLMethodsMixin
+        if environ is None:
+            environ = {}
         class Request(URLMethodsMixin):
             application_url = 'http://example.com:5432'
-        request = Request()
+            script_name = ''
+            def __init__(self, environ):
+                self.environ = environ
+        request = Request(environ)
         request.registry = self.config.registry
         return request
 
     def _registerContextURL(self, reg):
-        from pyramid.interfaces import IContextURL
+        with warnings.catch_warnings(record=True):
+            from pyramid.interfaces import IContextURL
         from zope.interface import Interface
         class DummyContextURL(object):
             def __init__(self, context, request):
@@ -29,109 +42,132 @@ class TestURLMethodsMixin(unittest.TestCase):
         reg.registerAdapter(DummyContextURL, (Interface, Interface),
                             IContextURL)
 
+    def _registerResourceURL(self, reg):
+        from pyramid.interfaces import IResourceURL
+        from zope.interface import Interface
+        class DummyResourceURL(object):
+            def __init__(self, context, request):
+                self.physical_path = '/context/'
+                self.virtual_path = '/context/'
+        reg.registerAdapter(DummyResourceURL, (Interface, Interface),
+                            IResourceURL)
+
     def test_resource_url_root_default(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         root = DummyContext()
         result = request.resource_url(root)
-        self.assertEqual(result, 'http://example.com/context/')
+        self.assertEqual(result, 'http://example.com:5432/context/')
 
     def test_resource_url_extra_args(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, 'this/theotherthing', 'that')
         self.assertEqual(
             result,
-            'http://example.com/context/this%2Ftheotherthing/that')
+            'http://example.com:5432/context/this%2Ftheotherthing/that')
 
     def test_resource_url_unicode_in_element_names(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
-        uc = unicode('La Pe\xc3\xb1a', 'utf-8')
+        self._registerResourceURL(request.registry)
+        uc = text_(b'La Pe\xc3\xb1a', 'utf-8')
         context = DummyContext()
         result = request.resource_url(context, uc)
         self.assertEqual(result,
-                     'http://example.com/context/La%20Pe%C3%B1a')
+                     'http://example.com:5432/context/La%20Pe%C3%B1a')
 
     def test_resource_url_at_sign_in_element_names(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, '@@myview')
         self.assertEqual(result,
-                     'http://example.com/context/@@myview')
+                     'http://example.com:5432/context/@@myview')
 
     def test_resource_url_element_names_url_quoted(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, 'a b c')
-        self.assertEqual(result, 'http://example.com/context/a%20b%20c')
+        self.assertEqual(result, 'http://example.com:5432/context/a%20b%20c')
 
     def test_resource_url_with_query_dict(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
-        uc = unicode('La Pe\xc3\xb1a', 'utf-8')
+        uc = text_(b'La Pe\xc3\xb1a', 'utf-8')
         result = request.resource_url(context, 'a', query={'a':uc})
         self.assertEqual(result,
-                         'http://example.com/context/a?a=La+Pe%C3%B1a')
+                         'http://example.com:5432/context/a?a=La+Pe%C3%B1a')
 
     def test_resource_url_with_query_seq(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
-        uc = unicode('La Pe\xc3\xb1a', 'utf-8')
+        uc = text_(b'La Pe\xc3\xb1a', 'utf-8')
         result = request.resource_url(context, 'a', query=[('a', 'hi there'),
-                                                             ('b', uc)])
+                                                           ('b', uc)])
         self.assertEqual(result,
-                     'http://example.com/context/a?a=hi+there&b=La+Pe%C3%B1a')
+            'http://example.com:5432/context/a?a=hi+there&b=La+Pe%C3%B1a')
+
+    def test_resource_url_with_query_empty(self):
+        request = self._makeOne()
+        self._registerResourceURL(request.registry)
+        context = DummyContext()
+        result = request.resource_url(context, 'a', query=[])
+        self.assertEqual(result,
+            'http://example.com:5432/context/a')
 
     def test_resource_url_anchor_is_after_root_when_no_elements(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, anchor='a')
         self.assertEqual(result,
-                         'http://example.com/context/#a')
+                         'http://example.com:5432/context/#a')
 
     def test_resource_url_anchor_is_after_elements_when_no_qs(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, 'a', anchor='b')
         self.assertEqual(result,
-                         'http://example.com/context/a#b')
+                         'http://example.com:5432/context/a#b')
 
     def test_resource_url_anchor_is_after_qs_when_qs_is_present(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, 'a', 
                                       query={'b':'c'}, anchor='d')
         self.assertEqual(result,
-                         'http://example.com/context/a?b=c#d')
+                         'http://example.com:5432/context/a?b=c#d')
 
     def test_resource_url_anchor_is_encoded_utf8_if_unicode(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
-        uc = unicode('La Pe\xc3\xb1a', 'utf-8') 
+        uc = text_(b'La Pe\xc3\xb1a', 'utf-8') 
         result = request.resource_url(context, anchor=uc)
-        self.assertEqual(result,
-                         'http://example.com/context/#La Pe\xc3\xb1a')
+        self.assertEqual(
+            result,
+            native_(
+                text_(b'http://example.com:5432/context/#La Pe\xc3\xb1a',
+                      'utf-8'),
+                'utf-8')
+            )
 
     def test_resource_url_anchor_is_not_urlencoded(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         context = DummyContext()
         result = request.resource_url(context, anchor=' /#')
         self.assertEqual(result,
-                         'http://example.com/context/# /#')
+                         'http://example.com:5432/context/# /#')
 
-    def test_resource_url_no_IContextURL_registered(self):
-        # falls back to TraversalContextURL
+    def test_resource_url_no_IResourceURL_registered(self):
+        # falls back to ResourceURL
         root = DummyContext()
         root.__name__ = ''
         root.__parent__ = None
@@ -142,12 +178,98 @@ class TestURLMethodsMixin(unittest.TestCase):
 
     def test_resource_url_no_registry_on_request(self):
         request = self._makeOne()
-        self._registerContextURL(request.registry)
+        self._registerResourceURL(request.registry)
         del request.registry
         root = DummyContext()
         result = request.resource_url(root)
-        self.assertEqual(result, 'http://example.com/context/')
+        self.assertEqual(result, 'http://example.com:5432/context/')
 
+    def test_resource_url_finds_IContextURL(self):
+        request = self._makeOne()
+        self._registerContextURL(request.registry)
+        root = DummyContext()
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            result = request.resource_url(root)
+            self.assertEqual(len(w), 1)
+        self.assertEqual(result, 'http://example.com/context/')
+        
+    def test_resource_url_with_app_url(self):
+        request = self._makeOne()
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_url(root, app_url='http://somewhere.com')
+        self.assertEqual(result, 'http://somewhere.com/context/')
+
+    def test_resource_url_with_scheme(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_url(root, scheme='https')
+        self.assertEqual(result, 'https://example.com/context/')
+
+    def test_resource_url_with_host(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_url(root, host='someotherhost.com')
+        self.assertEqual(result, 'http://someotherhost.com:8080/context/')
+
+    def test_resource_url_with_port(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_url(root, port='8181')
+        self.assertEqual(result, 'http://example.com:8181/context/')
+
+    def test_resource_url_with_local_url(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        def resource_url(req, info):
+            self.assertEqual(req, request)
+            self.assertEqual(info['virtual_path'], '/context/')
+            self.assertEqual(info['physical_path'], '/context/')
+            self.assertEqual(info['app_url'], 'http://example.com:5432')
+            return 'http://example.com/contextabc/'
+        root.__resource_url__ = resource_url
+        result = request.resource_url(root)
+        self.assertEqual(result, 'http://example.com/contextabc/')
+        
+    def test_resource_path(self):
+        request = self._makeOne()
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_path(root)
+        self.assertEqual(result, '/context/')
+
+    def test_resource_path_kwarg(self):
+        request = self._makeOne()
+        self._registerResourceURL(request.registry)
+        root = DummyContext()
+        result = request.resource_path(root, anchor='abc')
+        self.assertEqual(result, '/context/#abc')
+        
     def test_route_url_with_elements(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -172,28 +294,42 @@ class TestURLMethodsMixin(unittest.TestCase):
         mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.route_url('flub', a=1, b=2, c=3, _query={'a':1},
-                                   _anchor=u"foo")
+                                   _anchor=text_(b"foo"))
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3?a=1#foo')
 
-    def test_route_url_with_anchor_string(self):
+    def test_route_url_with_anchor_binary(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
         mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
         request.registry.registerUtility(mapper, IRoutesMapper)
-        result = request.route_url('flub', _anchor="La Pe\xc3\xb1a")
-        self.assertEqual(result,
-                         'http://example.com:5432/1/2/3#La Pe\xc3\xb1a')
+        result = request.route_url('flub', _anchor=b"La Pe\xc3\xb1a")
+
+        self.assertEqual(
+            result,
+            native_(
+                text_(
+                    b'http://example.com:5432/1/2/3#La Pe\xc3\xb1a',
+                    'utf-8'),
+                'utf-8')
+            )
 
     def test_route_url_with_anchor_unicode(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
         mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
         request.registry.registerUtility(mapper, IRoutesMapper)
-        anchor = unicode('La Pe\xc3\xb1a', 'utf-8')
+        anchor = text_(b'La Pe\xc3\xb1a', 'utf-8')
         result = request.route_url('flub', _anchor=anchor)
-        self.assertEqual(result,
-                         'http://example.com:5432/1/2/3#La Pe\xc3\xb1a')
+
+        self.assertEqual(
+            result,
+            native_(
+                text_(
+                    b'http://example.com:5432/1/2/3#La Pe\xc3\xb1a',
+                    'utf-8'),
+                'utf-8')
+            )
 
     def test_route_url_with_query(self):
         from pyramid.interfaces import IRoutesMapper
@@ -204,6 +340,15 @@ class TestURLMethodsMixin(unittest.TestCase):
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3?q=1')
 
+    def test_route_url_with_empty_query(self):
+        from pyramid.interfaces import IRoutesMapper
+        request = self._makeOne()
+        mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.route_url('flub', _query={})
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3')
+
     def test_route_url_with_app_url(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -213,6 +358,47 @@ class TestURLMethodsMixin(unittest.TestCase):
         self.assertEqual(result,
                          'http://example2.com/1/2/3')
 
+    def test_route_url_with_host(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'5432',
+            }
+        request = self._makeOne(environ)
+        mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.route_url('flub', _host='someotherhost.com')
+        self.assertEqual(result,
+                         'http://someotherhost.com:5432/1/2/3')
+
+    def test_route_url_with_port(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'5432',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.route_url('flub', _port='8080')
+        self.assertEqual(result,
+                         'http://example.com:8080/1/2/3')
+
+    def test_route_url_with_scheme(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'5432',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.route_url('flub', _scheme='https')
+        self.assertEqual(result,
+                         'https://example.com/1/2/3')
+        
     def test_route_url_generation_error(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -281,7 +467,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.matchdict = {}
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.current_route_url('extra1', 'extra2', _query={'a':1},
-                                           _anchor=u"foo")
+                                           _anchor=text_(b"foo"))
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3/extra1/extra2?a=1#foo')
 
@@ -294,7 +480,8 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.matchdict = {}
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.current_route_url('extra1', 'extra2', _query={'a':1},
-                                           _anchor=u"foo", _route_name='bar')
+                                           _anchor=text_(b"foo"),
+                                           _route_name='bar')
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3/extra1/extra2?a=1#foo')
 
@@ -308,7 +495,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.script_name = '/script_name'
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.current_route_path('extra1', 'extra2', _query={'a':1},
-                                            _anchor=u"foo")
+                                            _anchor=text_(b"foo"))
         self.assertEqual(result, '/script_name/1/2/3/extra1/extra2?a=1#foo')
         
     def test_route_path_with_elements(self):
@@ -319,7 +506,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.script_name = ''
         result = request.route_path('flub', 'extra1', 'extra2',
                                     a=1, b=2, c=3, _query={'a':1},
-                                    _anchor=u"foo")
+                                    _anchor=text_(b"foo"))
         self.assertEqual(result, '/1/2/3/extra1/extra2?a=1#foo')
 
     def test_route_path_with_script_name(self):
@@ -330,7 +517,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.route_path('flub', 'extra1', 'extra2',
                                     a=1, b=2, c=3, _query={'a':1},
-                                    _anchor=u"foo")
+                                    _anchor=text_(b"foo"))
         self.assertEqual(result, '/foo/1/2/3/extra1/extra2?a=1#foo')
         
     def test_static_url_staticurlinfo_notfound(self):
@@ -346,7 +533,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         abspath = makeabs('static', 'foo.css')
         result = request.static_url(abspath)
         self.assertEqual(result, 'abc')
-        self.assertEqual(info.args, ('/static/foo.css', request, {}))
+        self.assertEqual(info.args, (makeabs('static', 'foo.css'), request, {}))
         request = self._makeOne()
 
     def test_static_url_found_rel(self):
@@ -406,7 +593,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         abspath = makeabs('static', 'foo.css')
         result = request.static_path(abspath)
         self.assertEqual(result, 'abc')
-        self.assertEqual(info.args, ('/static/foo.css', request,
+        self.assertEqual(info.args, (makeabs('static', 'foo.css'), request,
                                      {'_app_url':'/foo'})
                          )
 
@@ -449,6 +636,168 @@ class TestURLMethodsMixin(unittest.TestCase):
                           {'_app_url':'/foo'})
                          )
 
+    def test_partial_application_url_with_http_host_default_port_http(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'HTTP_HOST':'example.com:80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'http://example.com')
+
+    def test_partial_application_url_with_http_host_default_port_https(self):
+        environ = {
+            'wsgi.url_scheme':'https',
+            'HTTP_HOST':'example.com:443',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'https://example.com')
+
+    def test_partial_application_url_with_http_host_nondefault_port_http(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'HTTP_HOST':'example.com:8080',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'http://example.com:8080')
+
+    def test_partial_application_url_with_http_host_nondefault_port_https(self):
+        environ = {
+            'wsgi.url_scheme':'https',
+            'HTTP_HOST':'example.com:4443',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'https://example.com:4443')
+
+    def test_partial_application_url_with_http_host_no_colon(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'HTTP_HOST':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'http://example.com')
+
+    def test_partial_application_url_no_http_host(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url()
+        self.assertEqual(result, 'http://example.com')
+        
+    def test_partial_application_replace_port(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(port=8080)
+        self.assertEqual(result, 'http://example.com:8080')
+
+    def test_partial_application_replace_scheme_https_special_case(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(scheme='https')
+        self.assertEqual(result, 'https://example.com')
+
+    def test_partial_application_replace_scheme_https_special_case_avoid(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(scheme='https', port='8080')
+        self.assertEqual(result, 'https://example.com:8080')
+
+    def test_partial_application_replace_scheme_http_special_case(self):
+        environ = {
+            'wsgi.url_scheme':'https',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'8080',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(scheme='http')
+        self.assertEqual(result, 'http://example.com')
+
+    def test_partial_application_replace_scheme_http_special_case_avoid(self):
+        environ = {
+            'wsgi.url_scheme':'https',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'8000',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(scheme='http', port='8080')
+        self.assertEqual(result, 'http://example.com:8080')
+        
+    def test_partial_application_replace_host_no_port(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(host='someotherhost.com')
+        self.assertEqual(result, 'http://someotherhost.com')
+
+    def test_partial_application_replace_host_with_port(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'8000',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(host='someotherhost.com:8080')
+        self.assertEqual(result, 'http://someotherhost.com:8080')
+
+    def test_partial_application_replace_host_and_port(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(host='someotherhost.com:8080',
+                                                  port='8000')
+        self.assertEqual(result, 'http://someotherhost.com:8000')
+
+    def test_partial_application_replace_host_port_and_scheme(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'80',
+            }
+        request = self._makeOne(environ)
+        result = request._partial_application_url(
+            host='someotherhost.com:8080',
+            port='8000',
+            scheme='https',
+            )
+        self.assertEqual(result, 'https://someotherhost.com:8000')
+        
+    def test_partial_application_url_with_custom_script_name(self):
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_NAME':'example.com',
+            'SERVER_PORT':'8000',
+            }
+        request = self._makeOne(environ)
+        request.script_name = '/abc'
+        result = request._partial_application_url()
+        self.assertEqual(result, 'http://example.com:8000/abc') 
+        
 class Test_route_url(unittest.TestCase):
     def _callFUT(self, route_name, request, *elements, **kw):
         from pyramid.url import route_url
@@ -654,5 +1003,7 @@ class DummyStaticURLInfo:
         return self.result
     
 def makeabs(*elements):
-    import os
-    return os.path.sep + os.path.sep.join(elements)
+    if WIN: # pragma: no cover
+        return r'c:\\' + os.path.sep.join(elements)
+    else:
+        return os.path.sep + os.path.sep.join(elements)
